// physics.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "resource.h"

#include <math.h>

#define MAX_LOADSTRING 100

typedef struct _tagVector
{
	float x;
	float y;
	float w;	// must be 1
} VECTOR, *LPVECTOR;


typedef union _tagMatrix3
{
	VECTOR v[3];
	float  m[3][3];
	struct {
		float m00, m01, m02;
		float m10, m11, m12;
		float m20, m21, m22;
	};
} MATRIX3, *LPMATRIX3;



#define OF_NORMAL	0
#define OF_STATIC	1


typedef struct _tagBoundingBox
{
	MATRIX3 pos;             /*/
     /                    \   *
     | cos(a) -sin(a)  x0 |   *
     | sin(a)  cos(a)  y0 |   *
     |   0       0      1 |   *
     \                    /  /*/

	float lenght;
	float width;
} BBOX, *LPBBOX;


struct _tagObject
{
	BBOX   bbox;
	VECTOR v;	// w   

	float m;
	float i;	//  

	DWORD flags;
};



#define NUMBER_OF_OBJECTS	7
_tagObject g_objects[NUMBER_OF_OBJECTS];
_tagObject g_objects_old[NUMBER_OF_OBJECTS];


#define vset(v, x0, y0, w0) {v.x = x0; v.y = y0; v.w = w0;}


void dump(_tagObject *pObject)
{
	FILE *file = fopen("dump.txt", "a");
	
	fprintf(file, "object dump:\n");
	fprintf(file, "M =   %g\n", pObject->m);
	fprintf(file, "I =   %g\n", pObject->i);
	fprintf(file, "V =   {%.5f, %.5f, %.5f}\n", pObject->v.x, pObject->v.y, pObject->v.w);
	fprintf(file, "pos = {%.5f, %.5f}\n", pObject->bbox.pos.m02, pObject->bbox.pos.m12);


	fclose(file);
}



__inline MATRIX3* mkidentity(LPMATRIX3 matrix)
{
	matrix->m00 = 1; matrix->m01 = 0; matrix->m02 = 0;
	matrix->m10 = 0; matrix->m11 = 1; matrix->m12 = 0;
	matrix->m20 = 0; matrix->m21 = 0; matrix->m22 = 1;

	return matrix;
} 

__inline MATRIX3* mkrotate(LPMATRIX3 matrix, float a)
{
	float s = sinf(a), c = cosf(a);

	matrix->m00 =  c; matrix->m01 = -s; matrix->m02 = 0;
	matrix->m10 =  s; matrix->m11 =  c; matrix->m12 = 0;
	matrix->m20 =  0; matrix->m21 =  0; matrix->m22 = 1;

	return matrix;
} 

__inline MATRIX3* mktranslate(LPMATRIX3 matrix, const LPVECTOR delta)
{
	matrix->m00 = 1; matrix->m01 = 0; matrix->m02 = delta->x;
	matrix->m10 = 0; matrix->m11 = 1; matrix->m12 = delta->y;
	matrix->m20 = 0; matrix->m21 = 0; matrix->m22 = 1;

	return matrix;
}
 
__inline MATRIX3* mktranslate(LPMATRIX3 matrix, float dx, float dy)
{
	matrix->m00 = 1; matrix->m01 = 0; matrix->m02 = dx;
	matrix->m10 = 0; matrix->m11 = 1; matrix->m12 = dy;
	matrix->m20 = 0; matrix->m21 = 0; matrix->m22 = 1;

	return matrix;
} 

__inline MATRIX3* mktransform(LPMATRIX3 matrix, float x0, float y0, float a)
{
	float s = sinf(a), c = cosf(a);

	matrix->m00 =  c; matrix->m01 = -s; matrix->m02 = x0;
	matrix->m10 =  s; matrix->m11 =  c; matrix->m12 = y0;
	matrix->m20 =  0; matrix->m21 =  0; matrix->m22 =  1;

	return matrix;
} 

// delta->w  -  rotation angle
__inline MATRIX3* mktransform(LPMATRIX3 matrix, const LPVECTOR delta)
{
	float s = sinf(delta->w), c = cosf(delta->w);

	matrix->m00 =  c; matrix->m01 = -s; matrix->m02 = delta->x;
	matrix->m10 =  s; matrix->m11 =  c; matrix->m12 = delta->y;
	matrix->m20 =  0; matrix->m21 =  0; matrix->m22 =  1;

	return matrix;
} 

__inline VECTOR* copy(VECTOR *dst, const LPVECTOR src)
{
	dst->x = src->x;
	dst->y = src->y;
	dst->w = src->w;
	return dst;
}

__inline MATRIX3* copy(LPMATRIX3 dst, const LPMATRIX3 src)
{
	dst->m00 = src->m00; dst->m01 = src->m01; dst->m02 = src->m02;
	dst->m10 = src->m10; dst->m11 = src->m11; dst->m12 = src->m12;
	dst->m20 = src->m20; dst->m21 = src->m21; dst->m22 = src->m22;
	return dst;
}

__inline VECTOR* fcopy(LPVECTOR dst, const LPVECTOR src)	// fast
{
	dst->x = src->x;
	dst->y = src->y;
	return dst;
}

__inline MATRIX3* fcopy(LPMATRIX3 dst, const LPMATRIX3 src)
{
	dst->m00 = src->m00; dst->m01 = src->m01; dst->m02 = src->m02;
	dst->m10 = src->m10; dst->m11 = src->m11; dst->m12 = src->m12;
	return dst;
}


__inline MATRIX3* Multiply(const LPMATRIX3 source, const LPMATRIX3 mult, LPMATRIX3 product)
{
	product->m00 = mult->m00*source->m00 + mult->m10*source->m01 + mult->m20*source->m02;
	product->m01 = mult->m01*source->m00 + mult->m11*source->m01 + mult->m21*source->m02;
	product->m02 = mult->m02*source->m00 + mult->m12*source->m01 + mult->m22*source->m02;
	product->m10 = mult->m00*source->m10 + mult->m10*source->m11 + mult->m20*source->m12;
	product->m11 = mult->m01*source->m10 + mult->m11*source->m11 + mult->m21*source->m12;
	product->m12 = mult->m02*source->m10 + mult->m12*source->m11 + mult->m22*source->m12;
	product->m20 = mult->m00*source->m20 + mult->m10*source->m21 + mult->m20*source->m22;
	product->m21 = mult->m01*source->m20 + mult->m11*source->m21 + mult->m21*source->m22;
	product->m22 = mult->m02*source->m20 + mult->m12*source->m21 + mult->m22*source->m22;

	return product;
}

__inline VECTOR* Multiply(const LPMATRIX3 matrix, const LPVECTOR source, LPVECTOR product)
{
	product->x = matrix->m00*source->x + matrix->m01*source->y + matrix->m02*source->w;
	product->y = matrix->m10*source->x + matrix->m11*source->y + matrix->m12*source->w;
	product->w = matrix->m20*source->x + matrix->m21*source->y + matrix->m22*source->w;

	return product;
}

__inline VECTOR* multiply(const LPVECTOR source, LPVECTOR product, float n)
{
	product->x = source->x * n;
	product->y = source->y * n;
	product->w = source->w * n;

	return product;
}

__inline float fmultiply(const LPVECTOR v1, const LPVECTOR v2)
{
	return v1->x *v2->x + v1->y *v2->y;
}

__inline float multiply(const LPVECTOR v1, const LPVECTOR v2)
{
	return v1->x *v2->x + v1->y *v2->y + v1->w *v2->w;
}

__inline VECTOR* Transform(const LPMATRIX3 matrix, const LPVECTOR source, LPVECTOR product)	// fast multiply
{
	product->x = matrix->m00*source->x + matrix->m01*source->y + matrix->m02;
	product->y = matrix->m10*source->x + matrix->m11*source->y + matrix->m12;

	return product;
}



#define INTERSECT_NONE	0
#define INTERSECT_OBJ1	1
#define INTERSECT_OBJ2	2
#define INTERSECT_UNKN	8

/*********************\
colpoint:
	m00, m01, m02;  }  
	m10, m11, m12;  } 
	m20, m21, m22;
\*********************/
int intersect(_tagObject *pObj1, _tagObject *pObj2, LPMATRIX3 colpoint)
{
	float R1  = 2 * sqrtf(
		(pObj1->bbox.pos.m02-pObj2->bbox.pos.m02)*(pObj1->bbox.pos.m02-pObj2->bbox.pos.m02) + 
		(pObj1->bbox.pos.m12-pObj2->bbox.pos.m12)*(pObj1->bbox.pos.m12-pObj2->bbox.pos.m12) );

	float R2 = sqrtf( pObj1->bbox.lenght*pObj1->bbox.lenght + pObj1->bbox.width*pObj1->bbox.width ) +
		sqrtf( pObj2->bbox.lenght*pObj2->bbox.lenght + pObj2->bbox.width*pObj2->bbox.width );

	if (R1 > R2) return INTERSECT_NONE;
	

	///////////////////////////////////////////////////////////////////
	// exact intersection testing

	VECTOR verts1[4] = { 
		{-0.5f*pObj1->bbox.lenght, -0.5f*pObj1->bbox.width, 1},
		{ 0.5f*pObj1->bbox.lenght, -0.5f*pObj1->bbox.width, 1},
		{ 0.5f*pObj1->bbox.lenght,  0.5f*pObj1->bbox.width, 1},
		{-0.5f*pObj1->bbox.lenght,  0.5f*pObj1->bbox.width, 1}
	};

	VECTOR verts2[4] = { 
		{-0.5f*pObj2->bbox.lenght, -0.5f*pObj2->bbox.width, 1},
		{ 0.5f*pObj2->bbox.lenght, -0.5f*pObj2->bbox.width, 1},
		{ 0.5f*pObj2->bbox.lenght,  0.5f*pObj2->bbox.width, 1},
		{-0.5f*pObj2->bbox.lenght,  0.5f*pObj2->bbox.width, 1}
	};

	VECTOR world = {1, 1, 1};
	int i, j;
	for (i = 0; i < 4; i++)
	{
		fcopy(verts1 + (i%4), Transform(&pObj1->bbox.pos, verts1 + (i%4), &world));
		fcopy(verts2 + (i%4), Transform(&pObj2->bbox.pos, verts2 + (i%4), &world));
	}

	float x1, y1, xx1, yy1; 
	float x2, y2, xx2, yy2;
	float det, t1, t2;
	for (i = 0; i < 4; i++)	for (j = 0; j < 4; j++)
	{
		x1 = verts1[i].x;
		xx1 = verts1[(i+1)%4].x;
		y1 = verts1[i].y;
		yy1 = verts1[(i+1)%4].y;
		x2 = verts2[j].x;
		xx2 = verts2[(j+1)%4].x;
		y2 = verts2[j].y;
		yy2 = verts2[(j+1)%4].y;

		if (0 != (det = (x2 - xx2)*(y1 - yy1) - (x1 - xx1)*(y2 - yy2)))
		{
			t2 = (xx1*(y2 - y1) + x2*(y1 - yy1) + x1*(yy1 - y2)) / det;
			if (t2 < 0 || t2 > 1) 
				continue;

			t1 = (xx2*(y2 - y1) + x1*(yy2 - y2) + x2*(y1 - yy2)) / det;
			if (t1 < 0 || t1 > 1) 
				continue;

			int result = INTERSECT_UNKN;

			if (colpoint)
			{
				colpoint->v[0].x = x1 + (xx1 - x1) * t1;
				colpoint->v[0].y = y1 + (yy1 - y1) * t1;

				if (fabsf(t1 - 0.5f) < fabsf(t2 - 0.5f))
				{
					result = INTERSECT_OBJ1;
					if (fabsf(xx1 - x1) > fabsf(yy1 - y1))
					{
						colpoint->v[1].y = -1.0;
						colpoint->v[1].x = (yy1 - y1) / (xx1 - x1);
					}
					else
					{
						colpoint->v[1].x = -1.0;
						colpoint->v[1].y = (xx1 - x1) / (yy1 - y1);
					}

					if ((colpoint->v[1].x)*(yy1 - y1)-(colpoint->v[1].y)*(xx1 - x1) < 0)
					{
						colpoint->v[1].x = -colpoint->v[1].x;
						colpoint->v[1].y = -colpoint->v[1].y;
					}
				}
				else
				{
					result = INTERSECT_OBJ2;
					if (fabsf(xx2 - x2) > fabsf(yy2 - y2))
					{
						colpoint->v[1].y = -1.0;
						colpoint->v[1].x = (yy2 - y2) / (xx2 - x2);
					}
					else
					{
						colpoint->v[1].x = -1.0;
						colpoint->v[1].y = (xx2 - x2) / (yy2 - y2);
					}

					if ((colpoint->v[1].x)*(yy2 - y2)-(colpoint->v[1].y)*(xx2 - x2) < 0)
					{
						colpoint->v[1].x = -colpoint->v[1].x;
						colpoint->v[1].y = -colpoint->v[1].y;
					}
				}
			}

			return result;
		}
	}

	return INTERSECT_NONE;
}

_tagObject* impulse(_tagObject *pObj, LPVECTOR pImpulse, LPVECTOR pPivot)
{
	pObj->v.x += pImpulse->x / pObj->m;
	pObj->v.y += pImpulse->y / pObj->m;

	pObj->v.w += ((pPivot->x-pObj->bbox.pos.m02)*pImpulse->y - 
		(pPivot->y-pObj->bbox.pos.m12)*pImpulse->x) / pObj->i;

	return pObj; 
}


/***********************\

  l0 - wc=0
  l1 - wc!=0
  .
  .
  .
  
\************************/


/*

typedef struct _tasSolveParams
{
	union {
		VECTOR  vc;  //     
		struct {float vxc, vyc, wc;};
	};

	union {
		VECTOR  r0;  //    
		struct {float x0c, y0c, rc;};
	};

	float f0c;	// . 
	
	//  :
	float x0l, y0l, vxl, vyl, 
		nxl, nyl; // (nx, ny) -  
} TRACEPARAMS, *LPTRACEPARAMS;

// fabs(nyl) > fabs(nxl)
__inline float evaluate_ny(LPTRACEPARAMS ptp, float t)
{
	return t*(ptp->vxc - ptp->vxl) + ptp->x0c - ptp->x0l + 
		ptp->rc*cosf(ptp->f0c + t*ptp->wc) - (ptp->nxl*(t*(ptp->vyc - ptp->vyl) + 
		ptp->y0c - ptp->y0l + ptp->rc*sinf(ptp->f0c + t*ptp->wc)))/ptp->nyl;
}

// fabs(nxl) > fabs(nyl)
__inline float evaluate_nx(LPTRACEPARAMS ptp, float t)
{
	return t*(ptp->vyc - ptp->vyl) + ptp->y0c - ptp->y0l +
		ptp->rc*sinf(ptp->f0c + t*ptp->wc) - (ptp->nyl*(t*(ptp->vxc - ptp->vxl) + 
		ptp->x0c - ptp->x0l + ptp->rc*cosf(ptp->f0c + t*ptp->wc)))/ptp->nxl;
}

BOOL trace_l2(LPTRACEPARAMS ptp, float *t, float *n)
{
	
}

  */

float momentum(_tagObject *pObj)
{
	return pObj->i * pObj->v.w + 
		pObj->bbox.pos.m02*pObj->v.y - 
		pObj->bbox.pos.m12*pObj->v.x;
}

float energy(_tagObject *pObj)
{
	return 0.5f*(pObj->i * pObj->v.w*pObj->v.w + 
		pObj->m*(pObj->v.y*pObj->v.y + pObj->v.x*pObj->v.x));
}


float geta(float x, float y, _tagObject *o1, _tagObject *o2, LPVECTOR l)
{
	float tmp1 = y*(o1->bbox.pos.m02-l->x)-x*(o1->bbox.pos.m12-l->y);
	float tmp2 = y*(o2->bbox.pos.m02-l->x)-x*(o2->bbox.pos.m12-l->y);
	float b = o1->m*o2->m*(o2->i*tmp1*tmp1+o1->i*tmp2*tmp2)+o1->i*o2->i*(o1->m+o2->m)*(x*x+y*y);
	float a = 2 * o1->i*o2->i*o1->m*o2->m * (
		( o1->v.x+o2->v.x-l->y*(o1->v.w+o2->v.w)+o1->bbox.pos.m12*o1->v.w+o2->bbox.pos.m12*o2->v.w )*x + 
		( o1->v.y+o2->v.y+l->x*(o1->v.w+o2->v.w)-o1->bbox.pos.m02*o1->v.w-o2->bbox.pos.m02*o2->v.w )*y);
	return - a/b;
}


int CheckCollisions(int *po1, int *po2, LPMATRIX3 pColPoint)
{
	int result;

	for (int i = 0; i < NUMBER_OF_OBJECTS; i++)
	for (int j = i+1; j < NUMBER_OF_OBJECTS; j++)
	{
		if (INTERSECT_NONE != (result = intersect(g_objects + i, g_objects + j, pColPoint)))
		{
			*po1 = i;
			*po2 = j;
			return result;
		}
	}

	return INTERSECT_NONE;
}

void MoveAll(float dt)
{
	MATRIX3 transform, temp;


	for (int i = 0; i < NUMBER_OF_OBJECTS; i++)
	{
		mktranslate(&transform, g_objects[i].v.x * dt, g_objects[i].v.y * dt);
		fcopy(&g_objects[i].bbox.pos, Multiply(&transform, &g_objects[i].bbox.pos, &temp));
		mkrotate(&transform, g_objects[i].v.w * dt);
		fcopy(&g_objects[i].bbox.pos, Multiply(&g_objects[i].bbox.pos, &transform, &temp));
	}
}

void TimeStep(float dt, HWND hWnd)
{
	float dt_a = 0, dt_b = dt;
	float dt_c = dt;
	float e = dt * 1e-3f;

	int i = -1, j = -1;
	MATRIX3 colpoint;

//	{
//		VECTOR gr = {0, dt*2, 0}, pos = {g_objects[0].bbox.pos.m02,g_objects[0].bbox.pos.m12, 0};
//
//		impulse(&g_objects[0], &gr, &pos);
//	}



	memcpy(g_objects_old, g_objects, sizeof(_tagObject) * NUMBER_OF_OBJECTS);


//	int count = 0;

	if (CheckCollisions(&i, &j, &colpoint))
	{
		int f = 0;
	}

	int intres = INTERSECT_NONE, intres_old;

	while (1)
	{
		MoveAll(dt_c);

		if (intres_old = CheckCollisions(&i, &j, &colpoint))
		{
			memcpy(g_objects, g_objects_old, sizeof(_tagObject) * NUMBER_OF_OBJECTS);
			dt_b = dt_c;
			intres = intres_old;
		}
		else
		{
			dt_a = dt_c;
			if (dt_b - dt_a < e) break;
		}

		dt_c = 0.5f * (dt_a + dt_b);

//		count++;
	}

	if (dt_c < 0.00001)
	{
		int f = 0;
	}

	if (intres)
	{
		float a = geta(colpoint.v[1].x, colpoint.v[1].y, g_objects + j, g_objects + i, &colpoint.v[0]);

		if (INTERSECT_OBJ2 == intres) 
		{
			int tmp = i;
			i = j;
			j = tmp;
		}

		VECTOR delta_p;
		multiply(&colpoint.v[1], &delta_p, a);

		if (fmultiply(&colpoint.v[1], &delta_p) > 0)
		{
			multiply(&delta_p, &delta_p, -1);
		}

//		float m1 = momentum(g_objects + i) + momentum(g_objects + j);
//		float e1 = energy(g_objects + i) + energy(g_objects + j);
		
		impulse(g_objects + i, &delta_p, &colpoint.v[0]);
		impulse(g_objects + j, multiply(&delta_p, &delta_p, -1), &colpoint.v[0]);

//		float m2 = momentum(g_objects + i) + momentum(g_objects + j);
//		float e2 = energy(g_objects + i) + energy(g_objects + j);
	}

	static float m_time = 0;
	m_time += dt_c;
	char str[32];
	sprintf(str, "time = %g; dt=%g", m_time, dt_c);
	SetWindowText(hWnd, str);
//	if (dt_c < dt) TimeStep(dt /*- dt_c*/, hWnd);
}

////////////////////////////////////////////////////////////////////////////////////////////

void DrawObject(HDC hdc, _tagObject *pObject)
{
	VECTOR vertexes[4] = { 
		{-0.5f*pObject->bbox.lenght, -0.5f*pObject->bbox.width, 1},
		{ 0.5f*pObject->bbox.lenght, -0.5f*pObject->bbox.width, 1},
		{ 0.5f*pObject->bbox.lenght,  0.5f*pObject->bbox.width, 1},
		{-0.5f*pObject->bbox.lenght,  0.5f*pObject->bbox.width, 1}
	};

	VECTOR world = {1, 1, 1};
	for (int i = 0; i < 5; i++)
	{
		Transform(&pObject->bbox.pos, vertexes + (i%4), &world);

		if (0 == i)
			MoveToEx(hdc, int(world.x), int(world.y), NULL);
		else
			LineTo(hdc, int(world.x), int(world.y));
	}
}



void Render(HDC hdc, RECT *rtViewport)
{
	FillRect(hdc, rtViewport, (HBRUSH) GetStockObject(WHITE_BRUSH));

	for (int i = 0; i < NUMBER_OF_OBJECTS; i++)
	{
		DrawObject(hdc, g_objects + i);
	}
}


void CreateObjects()
{
	memset(g_objects, 0, sizeof(_tagObject) * NUMBER_OF_OBJECTS);

	int z = 0;


	g_objects[z].bbox.lenght = 50;
	g_objects[z].bbox.width  = 50;
	g_objects[z].m           = 0.01f;
	g_objects[z].i = g_objects[z].m / 12.0f * (g_objects[z].bbox.lenght*g_objects[z].bbox.lenght + g_objects[z].bbox.width*g_objects[z].bbox.width);
	vset(g_objects[z].v, 10, 10, 0.0f);
	mktransform(&g_objects[z].bbox.pos, 100, 100, 0.7f);
	z++;
	if (NUMBER_OF_OBJECTS < z+1) return;

	g_objects[z].bbox.lenght = 50;
	g_objects[z].bbox.width  = 25;
	g_objects[z].m           = 0.005f;
	g_objects[z].i = g_objects[z].m / 12.0f * (g_objects[z].bbox.lenght*g_objects[z].bbox.lenght + g_objects[z].bbox.width*g_objects[z].bbox.width);
	vset(g_objects[z].v, 0, 0, 3.0f);
	mktransform(&g_objects[z].bbox.pos, 230, 150, 0.2f);
	z++;
	if (NUMBER_OF_OBJECTS < z+1) return;

	g_objects[z].bbox.lenght = 50;
	g_objects[z].bbox.width  = 25;
	g_objects[z].m           = 0.005f;
	g_objects[z].i = g_objects[z].m / 12.0f * (g_objects[z].bbox.lenght*g_objects[z].bbox.lenght + g_objects[z].bbox.width*g_objects[z].bbox.width);
	vset(g_objects[z].v, 0, -15, -1.0f);
	mktransform(&g_objects[z].bbox.pos, 230, 250, 0.2f);
	z++;
	if (NUMBER_OF_OBJECTS < z+1) return;

	
	g_objects[z].bbox.lenght = 400;
	g_objects[z].bbox.width  = 10;
	g_objects[z].m           = 1000;
	g_objects[z].i = g_objects[z].m / 12.0f * (g_objects[z].bbox.lenght*g_objects[z].bbox.lenght + g_objects[z].bbox.width*g_objects[z].bbox.width);
	vset(g_objects[z].v, 0, 0, 0.0f);
	mktransform(&g_objects[z].bbox.pos, 220, 10, 0.0f);
	z++;
	if (NUMBER_OF_OBJECTS < z+1) return;

	g_objects[z].bbox.lenght = 400;
	g_objects[z].bbox.width  = 10;
	g_objects[z].m           = 1000;
	g_objects[z].i = g_objects[z].m / 12.0f * (g_objects[z].bbox.lenght*g_objects[z].bbox.lenght + g_objects[z].bbox.width*g_objects[z].bbox.width);
	vset(g_objects[z].v, 0, 0, 0.0f);
	mktransform(&g_objects[z].bbox.pos, 220, 430, 0.0f);
	z++;
	if (NUMBER_OF_OBJECTS < z+1) return;

	g_objects[z].bbox.lenght = 10;
	g_objects[z].bbox.width  = 400;
	g_objects[z].m           = 1000;
	g_objects[z].i = g_objects[z].m / 12.0f * (g_objects[z].bbox.lenght*g_objects[z].bbox.lenght + g_objects[z].bbox.width*g_objects[z].bbox.width);
	vset(g_objects[z].v, 0, 0, 0.0f);
	mktransform(&g_objects[z].bbox.pos, 10, 220, 0.0f);
	z++;
	if (NUMBER_OF_OBJECTS < z+1) return;

	g_objects[z].bbox.lenght = 10;
	g_objects[z].bbox.width  = 400;
	g_objects[z].m           = 1000;
	g_objects[z].i = g_objects[z].m / 12.0f * (g_objects[z].bbox.lenght*g_objects[z].bbox.lenght + g_objects[z].bbox.width*g_objects[z].bbox.width);
	vset(g_objects[z].v, 0, 0, 0.0f);
	mktransform(&g_objects[z].bbox.pos, 430, 220, 0.0f);
	z++;
	if (NUMBER_OF_OBJECTS < z+1) return;
}



void SpeedTest(HWND hWnd)
{
	__int64 c1, c2, constant, min = 0;

	HANDLE process = GetCurrentProcess();
	HANDLE thread  = GetCurrentThread();

	DWORD old_class = GetPriorityClass(process);
	DWORD old_priority = GetThreadPriority(thread);

	SetPriorityClass(process, REALTIME_PRIORITY_CLASS);
	SetThreadPriority(thread, THREAD_PRIORITY_TIME_CRITICAL);

	for (int i = 0; i < 1700; i++)
	{
		QueryPerformanceCounter((LARGE_INTEGER*) &c1);
		
		////////////////////////////////////////
		// an empty function
	//	isintersect_empty(&g_objects[0], &g_objects[1]);
		///////////////////////////////////////

		QueryPerformanceCounter((LARGE_INTEGER*) &c2);
		constant = c2-c1;
		
		MATRIX3 tmp;
		
		QueryPerformanceCounter((LARGE_INTEGER*) &c1);

		///////////////////////////////////////
		// place your function here
		
			intersect(&g_objects[0], &g_objects[1], &tmp);

		//
		///////////////////////////////////////
	
		QueryPerformanceCounter((LARGE_INTEGER*) &c2);
	//	c2 -= constant;

	//	if (0 == i) min = c2-c1;
	//	if (c2-c1 < min) min = c2-c1;
		min += c2-c1;
	}

	SetThreadPriority(thread, old_priority);
	SetPriorityClass(process, old_class);

	__int64 freq, fps;
	QueryPerformanceFrequency((LARGE_INTEGER*) &freq);
	fps = 1700*freq/min;

	char str[128];
	sprintf(str, "%ld per second", fps);
	MessageBox(hWnd, str, "speed test", MB_ICONINFORMATION);
}



// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];								// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];								// The title bar text

// Foward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);

HWND g_hWnd;

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT  ps;
	HDC   hdc, hdc_back;
	HBITMAP hbmp, hbmp_old;
	RECT  rect;

	
	int wmId, wmEvent;
	TCHAR szHello[MAX_LOADSTRING];
	LoadString(hInst, IDS_HELLO, szHello, MAX_LOADSTRING);

	switch (message) 
	{
		case WM_CREATE:
	//		SetTimer(hWnd, 0, 10, NULL);
			break;
		case WM_TIMER:
			int i;
			for (i = 0; i < 1; i++) TimeStep(0.01f, hWnd);

			InvalidateRect(hWnd, NULL, FALSE);
			break;
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_ABOUT:
				   DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
				   break;
				case IDM_EXIT:
				   DestroyWindow(hWnd);
				   break;
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_PAINT:
			GetClientRect(hWnd, &rect);
			hdc = BeginPaint(hWnd, &ps);
			hdc_back = CreateCompatibleDC(hdc);
			hbmp = CreateCompatibleBitmap(hdc, rect.right, rect.bottom);
			hbmp_old = (HBITMAP) SelectObject(hdc_back, hbmp);

			Render(hdc_back, &rect);


			BitBlt(hdc, 0, 0, rect.right, rect.bottom, hdc_back, 0, 0, SRCCOPY);
			SelectObject(hdc_back, hbmp_old);
			DeleteObject(hbmp);
			DeleteDC(hdc_back);
			EndPaint(hWnd, &ps);
			break;
		case WM_RBUTTONDOWN:
			SpeedTest(hWnd);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	CreateObjects();

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_PHYSICS, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_PHYSICS);

	// Main message loop:
	while (1) 
	{
		if (PeekMessage(&msg, NULL, 0, 0, TRUE))
		{
			if (WM_QUIT == msg.message) break;

			if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
		else
		{
			SendMessage(g_hWnd, WM_TIMER, 0, 0);
		}
	}

	return msg.wParam;
}



//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage is only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_PHYSICS);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCSTR)IDC_PHYSICS;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   hInst = hInstance; // Store instance handle in our global variable

   g_hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!g_hWnd)
   {
      return FALSE;
   }

   ShowWindow(g_hWnd, nCmdShow);
   UpdateWindow(g_hWnd);

   return TRUE;
}


// Mesage handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
				return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}
